# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
The primary atomic composer configuration.

The appropriate entry in the 'releases' must be passed into
the AtomicComposer.composer method.

The code at the bottom maps+expands the paths &
commands to each of the releases, for ease of
configuration.
"""

# Check if we're running on RHEL6 and disable
# `mock --new-chroot` and `rpm-ostree --workdir-tmpfs`
import platform
dist = platform.dist()
rhel6 = dist[0] == 'redhat' and int(float(dist[1])) == 6

config = dict(
    releases={
        'f21-updates': {
            'name': 'f21-updates',
            'repo': 'updates',
            'version': '21',
            'arch': 'x86_64',

            # OSTree treefile configuration
            # https://github.com/projectatomic/rpm-ostree/blob/master/doc/treefile.md
            'tree': 'docker-host',
            'treefile': {
                'include': 'fedora-atomic-docker-host.json',
                'ref': 'fedora-atomic/f21/x86_64/updates/docker-host',
                'repos': ['fedora-21', 'updates'],
                'packages': [],
            },

            # The name of the mock container to build and maintain
            'mock': 'fedora-21-updates-x86_64',

            # The git branch to use in the `git_repo` for the parent
            # treefile & repo configurations
            'git_branch': 'f21',

            # Add or overwrite yum repository name:urls. This lets you
            # compose trees against your own repositories.
            'repos': {},
        },

        'f21-updates-testing': {
            'name': 'f21-updates-testing',
            'repo': 'updates-testing',
            'version': '21',
            'arch': 'x86_64',
            'tree': 'docker-host',
            'treefile': {
                'include': 'fedora-atomic-docker-host.json',
                'ref': 'fedora-atomic/f21/x86_64/updates-testing/docker-host',
                'repos': ['fedora-21', 'updates', 'updates-testing'],
            },
            'git_branch': 'f21',
            'mock': 'fedora-21-updates-testing-x86_64',
            'repos': {},
        },

        'rawhide': {
            'name': 'rawhide',
            'repo': 'rawhide',
            'version': 'rawhide',
            'arch': 'x86_64',
            'tree': 'docker-host',
            'treefile': {
                'include': 'fedora-atomic-docker-host.json',
                'ref': 'fedora-atomic/rawhide/x86_64/docker-host',
                'repos': ['rawhide'],
            },
            'git_branch': 'f21',
            'mock': 'fedora-rawhide-x86_64',
            'repos': {},
        },
    },

    # Package repositories to use in the mock container and ostree compose
    repos={
        'rawhide': 'https://mirrors.fedoraproject.org/metalink?repo={version}&arch={arch}',
        'fedora-{version}': 'https://mirrors.fedoraproject.org/metalink?repo=fedora-{version}&arch={arch}',
        'updates': 'https://mirrors.fedoraproject.org/metalink?repo=updates-released-f{version}&arch={arch}',
        'updates-testing': 'https://mirrors.fedoraproject.org/metalink?repo=updates-testing-f{version}&arch={arch}',
    },

    # Output directories
    prod_dir='/var/lib/fedora-atomic',
    work_dir='{prod_dir}/work',
    canonical_dir='{prod_dir}/{version}',
    output_dir='{work_dir}/{version}/{arch}/{tree}',
    log_dir='{work_dir}/logs/{version}/{arch}/{repo}/{tree}',

    # A list of other directories to mount in the mock container
    mount_dirs=[],

    # The git repo containing our parent treefiles and yum repos
    git_repo='https://git.fedorahosted.org/git/fedora-atomic.git',
    git_cache='{work_dir}/fedora-atomic.git',
    # Some branches contain custom .repo files that we don't want to use
    delete_repo_files=True,

    # Mock command
    mock_cmd='/usr/bin/mock -r {mock}',
    mock_clean=True,

    # OSTree commands
    ostree_init='/usr/bin/ostree --repo={output_dir} init --mode=archive-z2',
    ostree_compose='/usr/bin/rpm-ostree compose tree' +
            (rhel6 and ' ' or ' --workdir-tmpfs ') + '--repo={output_dir} %s',
    ostree_summary='/usr/bin/ostree --repo={output_dir} summary --update',

    # Bi-directional syncing
    #rsync_in_objs='/usr/bin/rsync -rvp --ignore-existing {canonical_dir}/objects/ {output_dir}/objects/',
    #rsync_in_rest='/usr/bin/rsync -rvp --exclude=objects/ {canonical_dir}/ {output_dir}/',
    #rsync_out_objs='/usr/bin/rsync -rvp --ignore-existing {output_dir}/objects/ {canonical_dir}/objects/',
    #rsync_out_rest='/usr/bin/rsync -rvp --exclude=objects/ {output_dir}/ {canonical_dir}/',

    # Define the config keys to map to each release, in the appropriate order
    map_to_release=('prod_dir', 'work_dir', 'output_dir', 'log_dir',
                    'git_repo', 'git_cache', 'mock_cmd', 'ostree_init',
                    'ostree_compose', 'ostree_summary', 'canonical_dir',
                    'repos', 'rsync_in_objs', 'rsync_in_rest', 'rsync_out_objs',
                    'rsync_out_rest', 'mount_dirs', 'mock_clean',
                    'delete_repo_files'),
)

# Map and expand variables to each release
for key in config.get('map_to_release', []):
    for name, release in config['releases'].items():
        if key in config:
            value = config[key]
            if isinstance(value, dict):
                release[key] = {}
                for k, v in value.items():
                    k = k.format(**release)
                    release[key][k] = v.format(**release)
            elif isinstance(value, (list, tuple)):
                release[key] = []
                for item in value:
                    release[key].append(item.format(**release))
            elif isinstance(value, bool):
                release[key] = value
            else:
                release[key] = value.format(**release)
